@@ -32,12 +32,14 @@
  * The Clutter Wayland compositor API is available since Clutter 1.8
  */
 
-#include <wayland-server.h>
-#include <clutter/wayland/clutter-wayland-surface.h>
+#ifndef __CLUTTER_WAYLAND_COMPOSITOR_H__
+#define __CLUTTER_WAYLAND_COMPOSITOR_H__
 
 G_BEGIN_DECLS
 
 void
-clutter_wayland_set_compositor_display (struct wl_display *display);
+clutter_wayland_set_compositor_display (void *display);
 
 G_END_DECLS
+
+#endif /* __CLUTTER_WAYLAND_COMPOSITOR_H__ */
