@@ -61,6 +61,8 @@ struct _ClutterStageX11
 
   ClutterStageX11State wm_state;
 
+  int scale_factor;
+
   guint is_foreign_xwin       : 1;
   guint fullscreening         : 1;
   guint is_cursor_visible     : 1;
@@ -68,6 +70,7 @@ struct _ClutterStageX11
   guint accept_focus          : 1;
   guint fullscreen_on_realize : 1;
   guint cursor_hidden_xfixes  : 1;
+  guint fixed_scale_factor    : 1;
 };
 
 struct _ClutterStageX11Class
